package be.iminds.ilabt.jfed.experimenter_gui.editor.views;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolBoxFilter;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolBoxFilterBuilder;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Testbed;
import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Label;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import org.controlsfx.control.CheckListView;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.URL;
import java.util.*;
import java.util.stream.Collectors;

public class EditTestbedToolBoxFilterDialog extends BorderPane {
    private final JFedGuiConfig config;
    private final TestbedInfoSource testbedInfoSource;

    private final ToolBoxFilter origToolBoxFilter;
    private final ToolBoxFilterBuilder toolBoxFilter;

    @FXML private Label selectedTestbedCountLabel;
    @FXML private CheckListView<TestbedWrapper> testbedIdList;

    private boolean cancel = true;

    public EditTestbedToolBoxFilterDialog(JFedGuiConfig config,
                                          TestbedInfoSource testbedInfoSource,
                                          ToolBoxFilter toolBoxFilter) {
        this.config = config;
        this.testbedInfoSource = testbedInfoSource;
        this.origToolBoxFilter = toolBoxFilter;
        this.toolBoxFilter = new ToolBoxFilterBuilder(toolBoxFilter);

        URL location = getClass().getResource("EditTestbedToolBoxFilterDialog.fxml");
        assert location != null;
        FXMLLoader fxmlLoader = new FXMLLoader(location, null);
        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @FXML
    private void initialize() {
        assert selectedTestbedCountLabel != null;
        assert testbedIdList != null;

        Set<TestbedWrapper> allTestbeds = config.getAllServersInVisibleResources()
                .stream()
                .map(Server::getTestbed)
                .map(TestbedWrapper::new)
                .collect(Collectors.toCollection(TreeSet::new));
        Set<TestbedWrapper> filteredTestbeds = allTestbeds
                .stream()
                .filter((tw) -> origToolBoxFilter.getTestbedIds().contains(tw.getTestbed().getId()))
                .collect(Collectors.toCollection(TreeSet::new));

        Comparator<TestbedWrapper> testbedWrapperComparable = (a, b) -> {
            boolean aFirst = filteredTestbeds.contains(a);
            boolean bFirst = filteredTestbeds.contains(b);
            if (aFirst != bFirst) {
                return aFirst ? -1 : 1;
            } else {
                return a.compareTo(b);
            }
        };

        List<TestbedWrapper> allTestbedsSorted = allTestbeds.stream()
                .sorted(testbedWrapperComparable)
                .collect(Collectors.toList());

        testbedIdList.setItems(
                FXCollections.observableArrayList(allTestbedsSorted)
        );

        for (TestbedWrapper tw: filteredTestbeds) {
            testbedIdList.getCheckModel().check(testbedIdList.getCheckModel().getItemIndex(tw));
        }

        testbedIdList.getCheckModel().getCheckedItems().addListener(
                (ListChangeListener<? super TestbedWrapper>) (e) -> updateCountLabel()
        );
        updateCountLabel();
    }

    private void updateCountLabel() {
        int count = testbedIdList.getCheckModel().getCheckedItems().size();
        selectedTestbedCountLabel.setText("("+count+" testbeds selected)");
    }

    public void onSaveAction(ActionEvent e) {
        cancel = false;
        Stage stage = (Stage) getScene().getWindow();
        stage.hide();
    }

    public void onCancelAction(ActionEvent e) {
        cancel = true;
        Stage stage = (Stage) getScene().getWindow();
        stage.hide();
    }

    public ToolBoxFilter getToolBoxFilter() {
        if (cancel)
            return origToolBoxFilter;
        else {
            toolBoxFilter.setTestbedIds(
                    testbedIdList.getCheckModel().getCheckedItems()
                            .stream()
                            .map(TestbedWrapper::getTestbed)
                            .map(Testbed::getId)
                            .collect(Collectors.toSet())
            );
            return toolBoxFilter.create();
        }
    }


    private static class TestbedWrapper implements Comparable {
        private final Testbed testbed;

        public TestbedWrapper(Testbed testbed) {
            this.testbed = testbed;
        }

        public Testbed getTestbed() {
            return testbed;
        }

        @Override
        public int compareTo(@NotNull Object o) {
            return toString().compareTo(o.toString());
        }

        @Override
        public String toString() {
            return testbed.getLongName();
        }
    }
}
